/*
 *  Albert Seward Copyright (C) 2005
 *  All Rights Reserved. Proprietary.
 *  alse7905@student.uu.se
 * 
 */

/* Graphical Mode 0
 * ----------------
 * 
 * About:
 * This mode has a very restricted memory use and is suitable for
 * setups without any external memory. 152x250x4bpp only consumes
 * 475 bytes of RAM if all blocks are stored in flash.
 * 
 * Usage:
 * The screen is divided into MODE0_COLUMNS*MODE0_ROWS blocks.
 * Each block is 8*MODE0_BLOCK_HEIGHT pixels.
 * Each pixel is 4 bits.
 * 
 * A block can be stored in either ram or program memory.
 * 
 * The screen array (MODE0_COLUMNS*MODE0_ROWS bytes) contains
 * byte pointers to each block to be shown. Use macros MODE0_PGMMAP() 
 * and MODE0_RAMMAP() to convert block index to byte pointer.
 * 
 * A maximum of 256 different blocks can be used. How these are
 * divided between RAM/PGM are specified by MODE0_PGM_BLOCK_COUNT
 * 
 * The three memory sections (screen array, block in RAM, block in PGM)
 * are given with mode0_mmap().
 * 
 * mode0() function is never called, but given as an argument to 
 * video_init();
 * 
 * A 16Mhz CPU gives maximum width of 19 blocks (152 pixels)
 * The maximum number of vertical blocks depend on the MODE0_BLOCK_HEIGHT
 * option, so MODE0_BLOCK_HEIGHT * MODE0_ROWS <= 256.
 * This gives a maximum resolution of 152x256x4bpp pixels at 16 Mhz.
 * 
 */

#ifndef _MODE0_H_
#define _MODE0_H_

#define MODE0_COLUMNS           19
#define MODE0_ROWS              24
#define MODE0_BLOCK_HEIGHT      10
#define MODE0_PGM_BLOCK_COUNT   240
#define MODE0_BLOCK_SIZE        (MODE0_BLOCK_HEIGHT * 4)

#define mode0_pgmmap(x)      (x)
#define mode0_rammap(x)      (MODE0_PGM_BLOCK_COUNT + (x))
#define mode0_is_rammap(x)   ((x) >= 240)

#define MODE0_XPOS 52         

#ifndef __ASM__
#include <avr/pgmspace.h>

uint8_t *_scrp;
uint8_t *_ramp;
PGM_P _pgmp;

uint8_t *__ramp;

#define mode0_get_block(x,y) (_scrp[(x)+(y)*MODE0_COLUMNS])
#define mode0_set_block(x,y, blk) do {_scrp[(x)+(y)*MODE0_COLUMNS] = (blk);} while(0)
#define mode0_cls()  memset(&mode0_get_block(0,0), ' ', MODE0_ROWS * MODE0_COLUMNS)

#define mode0_print(x, y, txt) \
({ \
	static char __c[] PROGMEM = (txt); \
	memcpy_P(&mode0_get_block((x),(y)), (__c), strlen_P(__c)); \
})

void mode0(uint8_t line);
static inline void mode0_mmap(uint8_t *screen, uint8_t *sprites, PGM_P pgm_bitmap) {
	_scrp = screen;
	_ramp = sprites;
	_pgmp = pgm_bitmap;
	__ramp = sprites - MODE0_PGM_BLOCK_COUNT * MODE0_BLOCK_SIZE;
}


#endif /* !__ASM__ */
#endif /* !_MODE0_H_ */
