/*
 *  Albert Seward Copyright (C) 2005
 *  All Rights Reserved. Proprietary.
 *  alse7905@student.uu.se
 * 
 * Floating sprites for Mode 0
 *
 *  TODO: Rewrite, I am not pleased with the need of 
 *        sprite_init and sprite id's
 */

#include <avr/pgmspace.h>
#include "video/video.h"
#include "video/mode0.h"

#include "sprite.h"

uint8_t s_blk[SPRITE_COUNT*2];
uint8_t s_x[SPRITE_COUNT*2];
uint8_t s_y[SPRITE_COUNT*2];

// TODO: rewrite with asm("swap");
static inline uint8_t swap_nibble(uint8_t x) {
	    return (x >> 4) | (x << 4);
}


void sprite_init() {
	uint8_t i;
	for(i = 0; i < SPRITE_COUNT*2; i++) {
		s_blk[i] = mode0_get_block(0, 0);
		s_x[i] = 0;
		s_y[i] = 0;
		
	}
}

void sprite_clear() {
	uint8_t i;
	
	for(i = 0; i < SPRITE_COUNT*2; i++) 
		mode0_set_block(s_x[i], s_y[i], s_blk[i]);
}

static inline void sprite_put_column(uint8_t id, uint8_t col, uint8_t y, uint8_t f) {
	uint8_t *sp, *blk0, *blk1, i, j, mf, row, *fig;
	
	row = DIV10(y);
	y = MOD10(y);
	id <<= 1;
	
	i = mode0_get_block(col, row);       	
	j = mode0_get_block(col, row+1);	
	
	s_x[id] = col;
	s_y[id] = row;
	s_blk[id] = i;
	
	s_x[id+1] = col;
	s_y[id+1] = row+1;
	s_blk[id+1] = j;
	
	mode0_set_block(col, row, mode0_rammap(id));	
	mode0_set_block(col, row+1, mode0_rammap(id+1));	
	
	sp = &_ramp[id * MODE0_BLOCK_SIZE]; 
	fig = (uint8_t *)&_pgmp[f * MODE0_BLOCK_SIZE];
	
	mf = 0;
	if(mode0_is_rammap(i)) {
		blk0 = &_ramp[(i-240) * MODE0_BLOCK_SIZE];
	} else {
		blk0 = (uint8_t *)&_pgmp[i * MODE0_BLOCK_SIZE];
		mf |= 1;
	}
		
	if(mode0_is_rammap(j)) {
		blk1 = &_ramp[(j-240) * MODE0_BLOCK_SIZE];
	} else {
		blk1 = (uint8_t *)&_pgmp[j * MODE0_BLOCK_SIZE];
		mf |= 2;
	}
	

	for(i = 0; i < (y*4); i++) {
		if(mf & 1) *(sp++) = pgm_read_byte(blk0++);
		else *(sp++) = *(blk0++);
	}
	
	for(i = 0; i < 40 ; i++)
			*(sp++) = pgm_read_byte(fig++);	
	blk1 += y * 4;
	
	for(i = 0; i < (10 - y)*4 ; i++) {
		if(mf & 2) *(sp++) = pgm_read_byte(blk1++);
		else *(sp++) = *(blk1++);
	}	
}

static inline void sprite_put_row(uint8_t id, uint8_t x, uint8_t row, uint8_t f) {

	uint8_t *sp, *fig, *blk0, *blk1, i, j, prex, pp0, pp1, mf, col;
		
	prex = (x & 7)>>1; // prex = 0,..,3
	col = x >> 3;

	id <<= 1;
	mf = 0;
		
	i = mode0_get_block(col, row);       	
	j = mode0_get_block(col+1, row);	
	
	s_x[id] = col;
	s_y[id] = row;
	s_blk[id] = i;
	s_x[id+1] = col+1;
	s_y[id+1] = row;
	s_blk[id+1] = j;
	
	if(col < MODE0_COLUMNS)	
	mode0_set_block(col, row, mode0_rammap(id));	
	
	if((x % 8) != 0 && (col < MODE0_COLUMNS-1))
		mode0_set_block(col+1, row, mode0_rammap(id+1));
	
	sp = &_ramp[id * MODE0_BLOCK_SIZE]; 
	fig = (uint8_t *)&_pgmp[f * MODE0_BLOCK_SIZE];
		
	if(mode0_is_rammap(i)) {
		blk0 = &_ramp[(i-240) * MODE0_BLOCK_SIZE];
	} else {
		blk0 = (uint8_t *)&_pgmp[i * MODE0_BLOCK_SIZE];
		mf |= 1;
	}
		
	if(mode0_is_rammap(j)) {
		blk1 = &_ramp[(j-240) * MODE0_BLOCK_SIZE];
	} else {
		blk1 = (uint8_t *)&_pgmp[j * MODE0_BLOCK_SIZE];
		mf |= 2;
	}
	       	        	
	
	if(ODD(x)) {
		for(i = 0; i < 10; i++) {
			
			pp0 = swap_nibble(pgm_read_byte(fig++));            
			if(mf & 1) {
				for(j = 0; j < prex; j++)
					*(sp++) = pgm_read_byte(blk0++);
				*(sp++) = (pgm_read_byte(blk0++) & 0xf0) | (pp0 & 0x0f); 
			} else {
				for(j = 0; j < prex; j++)
					*(sp++) = *(blk0++);
				*(sp++) = (*(blk0++) & 0xf0) | (pp0 & 0x0f); 
			}
                         
			blk0 += 3 - prex;
   
			for(j = 0; j < (3-prex); j++) {
				pp1 = swap_nibble(pgm_read_byte(fig++));
				*(sp++) = (pp0 & 0xf0) | (pp1 & 0x0f);
				pp0 = pp1;
			}
			
			sp += MODE0_BLOCK_SIZE - 4;
			
			for(j = 0; j < prex; j++) {
				pp1 = swap_nibble(pgm_read_byte(fig++));
				*(sp++) = (pp0 & 0xf0) | (pp1 & 0x0f);
				pp0 = pp1;
			}
			
			blk1 += prex;            
			if(mf & 2) {         
				*(sp++) = (pp0 & 0xf0) | (pgm_read_byte(blk1++) & 0x0f);
				for(j = 0; j < (3-prex); j++)
					*(sp++) = pgm_read_byte(blk1++);
			} else {
				*(sp++) = (pp0 & 0xf0) |  (*(blk1++) & 0x0f);
				for(j = 0; j < (3-prex); j++)
					*(sp++) = *(blk1++);
			}
			sp -= MODE0_BLOCK_SIZE;
		}
	} else { // Even
		for(i = 0; i < 10; i++) {
			if(mf & 1) {         				
				for(j = 0; j < prex; j++)
					*(sp++) = pgm_read_byte(blk0++);
			} else { 
				for(j = 0; j < prex; j++)
					*(sp++) = *(blk0++);
			}
				
			blk0 += 4 - prex;
			
			for(j = 0; j < (4-prex); j++)
				*(sp++) = pgm_read_byte(fig++);
			
			sp += MODE0_BLOCK_SIZE - 4;
			
			for(j = 0; j < prex; j++)
				*(sp++) = pgm_read_byte(fig++);
			
			blk1 += prex;
			
			if(mf & 2) {
				for(j = 0; j < (4-prex); j++)
					*(sp++) = pgm_read_byte(blk1++);
			} else {			
				for(j = 0; j < (4-prex); j++)
					*(sp++) = *(blk1++);
				
			}
			 
			sp -= MODE0_BLOCK_SIZE;
		}
	}       		
}


void sprite_put(uint8_t id, uint8_t x, uint8_t y, uint8_t f) {

	if(MOD10(y) == 0)
		sprite_put_row(id, x, DIV10(y), f);
	else
		sprite_put_column(id, DIV8(x), y, f);
    
	// TODO: Write support for !MOD10(y) && !MOD8(x)	       	
}
