/*
 *  Albert Seward Copyright (C) 2005
 *  All Rights Reserved. Proprietary.
 *  alse7905@student.uu.se
 * 
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/pgmspace.h>
#include <avr/delay.h>
#include "video.h"

#if SUPPORT_PAL == 1
monitor_t _PAL PROGMEM = __PAL;
#endif
#if SUPPORT_NTSC == 1
monitor_t _NTSC PROGMEM = __NTSC;
#endif

monitor_t *monitor;
mode_t *gmode;
prog_char *snd_file;
uint8_t snd_dur;

inline void sound_play(uint8_t *snd) 
{
	snd_file = snd;
	DDRD |= 0x08;
	TCNT2 = 0;
	snd_dur = 1;
}

inline void sync(void) 
{
	wsync = 0;
	loop_until_bit_is_set(wsync, 0);
}


SIGNAL(SIG_OVERFLOW1) 
{
	static unsigned char l0, l1, c0, c1;
	static unsigned char line;
	uint8_t tmp;
	
	line++;

	if(!c0) //kolikrat se puls opakuje
        {
		l0++;                                                         //prejdi na dalsi puls
  		if(l0 == pgm_read_byte(&monitor->csync0))
                {
                  	l0 = 0;             //neni to posledni?

                  	// Simple sound support. This is not the right place!
         		if(!(OCR2A = pgm_read_byte(snd_file++)))
				DDRD &= ~0x08;

		}

                //v l0 je cislo pulsu

		ICR1 = pgm_read_word(&monitor->timings[l0].width);  //jeho CELKOVA sirka
		c0 = pgm_read_byte(&monitor->timings[l0].times);    //kolikrat se bude opakovat
		line = 0xff;
		wsync=l0;

	}
	c0--;     //kolik je tam pulzu? asi hsync?

	if(!c1)
        {
		l1++;
		if(l1 == pgm_read_byte(&monitor->csync1)) l1 = 0;

		tmp = l1 + pgm_read_byte(&monitor->csync0);
		OCR1A = pgm_read_word(&monitor->timings[tmp].width);
		//tmp = l1 + pgm_read_byte(&monitor->csync0);
		c1 = pgm_read_byte(&monitor->timings[tmp].times);
	}
	c1--;

	if(((l0 == 2) | (l0 == 6)) && line != 0xff)
		gmode(line);                                 //vyflusni linku.

	// Disable pending interrupt. This is _very_ bad!
	// TIFR |= 1 << TOV1;
}

void video_init(monitor_t *mon, mode_t *gm) 
{
	monitor = mon;
	gmode = gm;
	
	DDRD = 1 << 5;

	// Video nibble output
	VIDEO_DDR = 0xf0;

	// Set on compare match, clear at TOP
	TCCR1A |= 1 << COM1A1 | 1 << COM1A0;

	// Counter 1 - Fast PWM 
	TCCR1A |= 1 << WGM11 | 0 << WGM10;            
	TCCR1B |= 1 << WGM13 | 1 << WGM12;
	
	// No prescaling
	TCCR1B |= 0 << CS12 | 0 << CS11 | 1 << CS10;  
	TIMSK1 |= 1 << TOIE1;
	
	ICR1 = PAL_LP;	
	OCR1A = PAL_SP;

DDRB |= 0x02;
}
