/*
 * Albert Seward Copyright (C) 2005
 * All Rights Reserved. Proprietary.
 * alse7905@student.uu.se
 * 
 */


#ifndef _VIDEO_H_
#define _VIDEO_H_

#define YES 1
#define NO  0

#define SUPPORT_PAL  YES
#define SUPPORT_NTSC NO    /* Broken! */

#define VIDEO_PORT              PORTD
#define VIDEO_DDR               DDRD
#define CPU_MHZ                 16.000000

#ifndef __ASM__

#include <avr/pgmspace.h>

struct timings {
	        uint16_t width;
	        uint8_t times;
} __attribute__((packed));

typedef struct {
	        uint8_t field1, field2;
	        uint8_t csync0, csync1;
	        struct timings timings[];
} const monitor_t PROGMEM;

typedef void (mode_t)(uint8_t);

//#define sound_init()  (TCCR2B = (1 << WGM21) | (1 << COM20) | (1 << CS22) | (1 << CS21) | (1 << CS20))
#define sound_init()  ({TCCR2A = 0x12;    TCCR2B = 0x07; })


volatile uint8_t wsync;

inline void sound_play(uint8_t *snd);

inline void sync(void);

void video_init(monitor_t *mon, mode_t *gm);
	

#if SUPPORT_PAL	== 1 /* 625 lines */
/* PAL Video Standard 
 * 
 * V-Sync timings
 *  DEP  __________            ___
 *    | |                 DFP |IFP|
 *    |_|    IEP       _______|   |
 * 
 *
 * H-Sync timings
 *                |\/||/\//\|
 *            BP  |    AP   | FP
 * _        ______|         |__
 *  |  SP  |
 *  |______|
 */
extern monitor_t _PAL PROGMEM; /* Ugly fix, but gcc does not behave as I would like to */
#define PAL ((monitor_t *)&_PAL)
#define PAL_LP          (int)(64*CPU_MHZ)   /* Nominal line period 64 us */
#define PAL_DEP         (int)(2.35*CPU_MHZ) /* Duration of equalizing pulses 2.35 s */
#define PAL_DFP         (int)(27.3*CPU_MHZ) /* Duration of field-synchronizing pulses 27.3 s */
#define PAL_SP          (int)(4.7*CPU_MHZ)  /* Synchronizing pulse 4.7 s */
// #define PAL_BP          (int)(5.8*CPU_MHZ)  /* PAL_BP - Back porch 5.8 us */
// #define PAL_AP          (int)(52*CPU_MHZ)   /* Active picture area 52 us */
// #define PAL_FP          (int)(1.5*CPU_MHZ)  /* Front porch 1.5 us */
#define __PAL  { \
	field1:         2, \
	field2:         6, \
	csync0:         9, \
	csync1:         13, \
        timings: { \
		/* Pulses total length */ \
		{PAL_LP / 2, 10}, \
		{PAL_LP, 40}, {PAL_LP, 241}, {PAL_LP, 305-241-40}, \
		{PAL_LP / 2, 14}, \
		{PAL_LP, 40}, {PAL_LP, 241}, {PAL_LP, 305-241-40}, \
		{PAL_LP / 2, 6}, \
		/* Low part of pulse, delayed one pulse */ \
                {PAL_DFP, 4}, {PAL_DEP, 5}, {PAL_SP, 25}, {PAL_SP, 255}, \
		{PAL_SP, 25}, {PAL_DEP, 5}, {PAL_DFP, 5}, {PAL_DEP, 5}, \
		{PAL_SP, 25}, {PAL_SP, 255}, {PAL_SP, 25}, {PAL_DEP, 5}, \
		{PAL_DFP, 1}} \
}
#endif /* SUPPORT_PAL */

#if SUPPORT_NTSC == 1 /* 525 lines */
extern monitor_t _NTSC PROGMEM; /* Ugly fix */
#define NTSC ((monitor_t *)&_NTSC)
#define NTSC_LP         (int)(63.5555*CPU_MHZ)   /* Nominal line period 63.5555 us */
#define NTSC_DEP        (int)(2.3*CPU_MHZ)  /* Duration of equalizing pulses 2.3 s */
#define NTSC_DFP        (int)(27.1*CPU_MHZ) /* Duration of field-synchronizing pulses 27.3 s */
#define NTSC_SP         (int)(4.7*CPU_MHZ)  /* Synchronizing pulse 4.7 s */

#define __NTSC  { \
	field1:         2, \
	field2:         6, \
	csync0:         9, \
	csync1:         13, \
        timings: { \
		/* Pulses total length */ \
		{NTSC_LP / 2, 12}, \
                {NTSC_LP, 6}, {NTSC_LP, 241}, {NTSC_LP, 253-241-6}, \
		{NTSC_LP / 2, 18}, \
		{NTSC_LP, 6+1}, {NTSC_LP, 241}, {NTSC_LP, 253-241-6}, \
		{NTSC_LP / 2, 6}, \
		/* Low part of pulse, delayed one pulse */ \
                {NTSC_DFP, 5}, {NTSC_DEP, 6}, \
		{NTSC_SP, 5+1}, {NTSC_SP, 253-5-5}, {NTSC_SP, 5}, \
		{NTSC_DEP, 6}, {NTSC_DFP, 6}, {NTSC_DEP, 6}, \
		{NTSC_SP, 5}, {NTSC_SP, 253-5-5}, {NTSC_SP, 5}, \
		{NTSC_DEP, 6}, {NTSC_DFP, 1}} \
}

#endif /* SUPPORT_NTSC */

#endif /* !__ASM__ */

#endif /* _VIDEO_H_ */
